import numpy as np
from config.core import config
from pipeline import category_prediction_pipeline
from processing.data_manager import load_dataset, save_pipeline
from sklearn.model_selection import train_test_split
from multiclass_model.custom_functions import custom_functions as cf
from processing.data_manager import load_json

def run_training() -> None:
    """Train the model."""

    # read training data
    data = load_dataset(file_name=config.app_config.training_data_file)
    map_target = load_json(file_name=config.app_config.json_file_target)

    ## to get numerical labels for the target
    data['Map_Product_Category'] = data.Product_Category.map(map_target)

    # divide train and test
    X_train, X_test, y_train, y_test = train_test_split(
        data[config.model_config.initial_features],  # predictors
        data[config.model_config.target],
        test_size=config.model_config.test_size,
        random_state=config.model_config.random_state,
    )
    X_train = cf.splitter(data = X_train)

    # fit model
    category_prediction_pipeline.fit(X_train, y_train)

    # persist trained model
    save_pipeline(pipeline_to_persist=category_prediction_pipeline)


if __name__ == "__main__":
    run_training()



from typing import List

import pandas as pd
import numpy as np
from sklearn.base import BaseEstimator, TransformerMixin

def splitter(data: pd.DataFrame)  -> pd.DataFrame:
    data[['TypeEnt','TypeEnt_number']] = data.Vendor_Code.str.split('-',expand=True)
    data[['GCL','GCL_number']] = data.GL_Code.str.split('-',expand=True)
    return data


class Mapper(BaseEstimator, TransformerMixin):
    """Categorical variable mapper."""

    def __init__(self, variables: List[str], mappings: dict):

        if not isinstance(variables, list):
            raise ValueError("variables should be a list")

        self.variables = variables
        self.mappings = mappings

    def fit(self, X: pd.DataFrame, y: pd.Series = None):
        # we need the fit statement to accomodate the sklearn pipeline
        return self

    def transform(self, X: pd.DataFrame) -> pd.DataFrame:
        X = X.copy()
        for feature in self.variables:
            X[feature] = X[feature].map(self.mappings)

        return X
    
class Custom_Fillna(BaseEstimator, TransformerMixin):
    
    def __init__(self, variables: List[str], fill_value: int):
        
        if not isinstance(variables, list):
            raise ValueError('variables should be a list')
            
        self.variables = variables
        self.fill_value = fill_value
        
    def fit(self, X : pd.DataFrame, y: pd.Series = None):
        # we need the fit statement to accomodate the sklearn pipeline
        return self

    def transform(self, X: pd.DataFrame):
        X = X.copy()
        for feature in self.variables:
            X[feature] = X[feature].fillna(self.fill_value)

        return X


from feature_engine.encoding import OrdinalEncoder
from feature_engine.selection import DropFeatures

from sklearn.multiclass import OneVsRestClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.pipeline import Pipeline

from multiclass_model.config.core import config

from multiclass_model.custom_functions import custom_functions as cf
from processing.data_manager import load_json
from config.core import config

TypeEnt_number_maps = load_json(file_name=config.app_config.json_file_TypeEnt)

category_prediction_pipeline = Pipeline([
    ('GCL_Code-cardinal-ordering',
     OrdinalEncoder(encoding_method='ordered', variables = config.model_config.ordinal_encode)
    ),
    
    ('TypeEnt_number_map_modes', 
    cf.Mapper(variables = config.model_config.mapper_encode, mappings = TypeEnt_number_maps)
    ),
    
    ('drop_features',
     DropFeatures(features_to_drop= config.model_config.drop_features)
    ),
    
    ('Fill_na',
    cf.Custom_Fillna(variables = config.model_config.fillna_features, fill_value = 0)
    ),
    
    ('KNN', OneVsRestClassifier(KNeighborsClassifier(n_neighbors = config.model_config.n_neighbors)))
    
])



from typing import List, Optional, Tuple

import numpy as np
import pandas as pd
from pydantic import BaseModel, ValidationError

from multiclass_model.config.core import config


class InvoiceDataSchema(BaseModel):
    Inv_Id: Optional[int]
    Vendor_Code: Optional[str]
    GL_Code: Optional[str]
    Inv_Amt:Optional[float]
    Item_Description: Optional[str]
    


class MultipleInvoiceDataSchema(BaseModel):
    inputs: List[InvoiceDataSchema]



import typing as t

import numpy as np
import pandas as pd

from multiclass_model import __version__ as _version
from multiclass_model.config.core import config
from multiclass_model.processing.data_manager import load_pipeline

pipeline_file_name = f"{config.app_config.pipeline_save_file}{_version}.pkl"
_classification_pipe = load_pipeline(file_name=pipeline_file_name)


def make_prediction(input_data: pd.DataFrame) -> dict:

    data = pd.DataFrame(input_data)
    results = {"predictions": None, "version": _version, }

    predictions = _classification_pipe.predict(data)

    results = {
        "predictions": [int(y_hat) for y_hat in predictions],  # type: ignore
        "version": _version,
    }

    return results
